package com.emarcotte.testrio;

import java.util.ArrayList;
import java.util.LinkedList;
import java.util.List;

import javax.media.opengl.GL;

import com.emarcotte.glfoo.utils.IndexedVertexBufferObject;
import com.sun.opengl.util.texture.Texture;

public class MD5Model {
	MD5Frame bindpos;

	MD5Frame currentFrame;
	List<MD5Animation> anims;

	List<MD5Mesh> meshes;
	int anim;
	Texture t;

	public MD5Model(int numJoints, int numMeshes) {
		bindpos = new MD5Frame(numJoints);
		meshes = new ArrayList<MD5Mesh>(numMeshes);
		anims = new ArrayList<MD5Animation>();
	}

	public void computeBindPosition() {
		for (MD5Mesh m : meshes) {
			m.update(bindpos);
		}
	}

	public void addAnimation(MD5Animation currentAnimation) {
		anims.add(currentAnimation);
	}

	public void addMesh(int i, MD5Mesh currentMesh) {
		meshes.add(i, currentMesh);
	}

	public void setBindPos() {
		currentFrame = bindpos;
	}

	public void updateAnimation() {
		currentFrame = anims.get(anim).getCurrentFrame();

		if (currentFrame == null)
			anims.get(anim).reset();

		currentFrame = anims.get(anim).getCurrentFrame();

		MD5Frame nextFrame = anims.get(anim).getNextFrame();

		// TODO: Do NOT USE BIND POS!! Should copy it and use some sort of
		// "currentState"
		bindpos.interpolate(currentFrame, nextFrame, anims.get(anim)
				.getFactor());

		for (MD5Mesh m : meshes) {
			m.update(bindpos);
		}
	}

	public void draw(GL gl) {
		gl.glPushMatrix();
		gl.glRotated(90, 0, 0, -1);
		gl.glRotated(90, 0, -1, 0);

		for (MD5Mesh m : meshes) {
			m.draw(gl);
			gl.glTranslated(50, 0, 0);
		}
		gl.glPopMatrix();
	}

	public void drawNormals(GL gl) {
		gl.glPushMatrix();
		gl.glRotated(90, 0, 0, -1);
		gl.glRotated(90, 0, -1, 0);

		for (MD5Mesh m : meshes) {
			m.drawNormals(gl);
			gl.glTranslated(50, 0, 0);
		}
		gl.glPopMatrix();
	}

	public List<IndexedVertexBufferObject> getMeshVBOs() {
		LinkedList<IndexedVertexBufferObject> vbos = new LinkedList<IndexedVertexBufferObject>();

		for (MD5Mesh m : meshes) {

			IndexedVertexBufferObject vbo = new IndexedVertexBufferObject(m
					.getIndices());

			vbo.setVertexPositions(m.getPositions(), GL.GL_TRIANGLES);
			vbo.setTextureCoordinates(m.getTexCoords(), 2);
			vbo.setNormalData(m.getNormals());

			vbos.add(vbo);
		}
		return vbos;
	}

	public void setBaseFrame(MD5Frame joints) {
		bindpos = joints;
	}

	public void nextAnim() {
		anim++;
		if (anim >= anims.size()) {
			anim = 0;
		}

	}

}
