package modelview;

import java.io.InputStream;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

import javax.media.opengl.GL;
import javax.media.opengl.glu.GLU;

import com.emarcotte.glfoo.ApplicationContext;
import com.emarcotte.glfoo.GraphicsDriver;
import com.emarcotte.glfoo.MainWindow;
import com.emarcotte.glfoo.Renderable;
import com.emarcotte.glfoo.RenderingContext;
import com.emarcotte.glfoo.implementation.ApplicationContextProvider;
import com.emarcotte.glfoo.implementation.DefaultCamera;
import com.emarcotte.glfoo.implementation.DefaultMouseHandler;
import com.emarcotte.glfoo.implementation.DefaultRenderingContext;
import com.emarcotte.glfoo.implementation.GraphicRenderer;
import com.emarcotte.glfoo.utils.GLShader;
import com.emarcotte.glfoo.utils.GLShaderProgram;
import com.emarcotte.glfoo.utils.IndexedVertexBufferObject;
import com.emarcotte.glfoo.utils.Light2;
import com.emarcotte.glfoo.utils.ResourceHandler;
import com.emarcotte.glfoo.utils.VertexBufferObject;
import com.emarcotte.testrio.MD5Loader;
import com.emarcotte.testrio.MD5Model;
import com.sun.opengl.util.texture.Texture;
import com.sun.opengl.util.texture.TextureIO;

public class Md5Test extends DefaultRenderingContext implements Renderable {

	Texture normal;
	Texture regular;
	private GLShaderProgram program;
	private List<IndexedVertexBufferObject> vbos;
	MD5Model m;
	Light2 l;

	public Md5Test(MainWindow w) {
		// BlurEffect effect = new BlurEffect();
		// PostProcessingRenderer renderer = new PostProcessingRenderer(effect);
		DefaultCamera dc = new DefaultCamera();
		dc.setZ(150);
		dc.setY(50);
		GraphicRenderer renderer = new GraphicRenderer(dc);
		addRenderer(renderer);
		renderer.addRenderable(this);
		w.addMouseHandler(new DefaultMouseHandler(dc));
		loadModelStuff();
	}

	/**
	 * Just loads models ugly nasty brute force to get things on the screen
	 */
	private void loadModelStuff() {
		MD5Loader loader = new MD5Loader();

		InputStream mesh = ResourceHandler.getHandler().getResourceAsStream(
				"boney/boney.md5mesh");

		Map<String, InputStream> animations;
		animations = new HashMap<String, InputStream>();
		InputStream anim = ResourceHandler.getHandler().getResourceAsStream(
				"boney/chest_pain1.md5anim");
		animations.put("boney/chest_pain1.md5anim", anim);

		loader.loadModel(mesh, animations);

		m = loader.getModel();
		vbos = m.getMeshVBOs();
	}

	@Override
	public void init(GraphicsDriver driver) {
		GL gl = driver.getGl();
		GLU glu = driver.getGlu();

		// load textures
		try {
			normal = TextureIO.newTexture(ResourceHandler.getHandler()
					.getResourceAsStream("bark0/bark0DOT3.jpg"), true,
					TextureIO.JPG);
			regular = TextureIO.newTexture(ResourceHandler.getHandler()
					.getResourceAsStream("tex.png"), true, TextureIO.JPG);
		} catch (Exception e) {
			e.printStackTrace();
		}

		// remove back faces
		gl.glEnable(GL.GL_CULL_FACE);
		gl.glFrontFace(GL.GL_CW);
		gl.glCullFace(GL.GL_BACK);

		// set up a light
		l = new Light2();
		l.setDirectional(true);
		l.setPosition(0, -100, 0);
		l.setAmbientColor(.5f, .5f, .5f);
		l.setDiffuseColor(1, 1, 1);
		l.setSpecularColor(1, 0, 0);
		l.applyLight(GL.GL_LIGHT0, gl);

		gl.glEnable(GL.GL_LIGHTING); // not needed shaders are running.
		gl.glEnable(GL.GL_LIGHT0); // not needed shaders are running.
		gl.glEnable(GL.GL_TEXTURE_2D); // probably not needed shaders are
		// running.

		// set up the shader stuff
		program = GLShaderProgram.createProgram(gl);

		GLShader vertexShader = GLShader.load(gl, glu,
				"shaders/outline_vert.glsl", GL.GL_VERTEX_SHADER);

		GLShader fragShader = GLShader.load(gl, glu,
				"shaders/outline_frag.glsl", GL.GL_FRAGMENT_SHADER);

		program.add(vertexShader);
		program.add(fragShader);
		program.link();
		program.enable();
		// set uniforms for the shader
		program.setUniformi("colorMap", 0);
		program.setUniformi("normalMap", 1);
		program.disable();

		for (IndexedVertexBufferObject v : vbos) {
			v.init(driver);
		}
	}

	public void draw(GraphicsDriver driver) {
		GL gl = driver.getGl();
		gl.glClearColor(0, 0, 0, 0);

		// setup multi texturing and bind textures.
		gl.glActiveTexture(GL.GL_TEXTURE0);
		regular.bind();
		gl.glActiveTexture(GL.GL_TEXTURE1);
		normal.bind();

		l.applyLight(GL.GL_LIGHT0, gl);
		// m.updateAnimation();
		m.setBindPos();

		gl.glPushMatrix();
		gl.glRotated(90, 0, 0, -1);
		gl.glRotated(90, 0, -1, 0);
		// use special shader and draw
		// program.enable();
		for (VertexBufferObject v : vbos) {
			v.draw(driver);
			gl.glTranslated(50, 0, 0);
		}
		// m.draw(driver.getGl());
		// program.disable();

		// draw normals?
		// m.drawNormals(gl);

		gl.glPushMatrix();
		// set textures back to nothing
		gl.glBindTexture(regular.getTarget(), 0);
		gl.glActiveTexture(0);
		gl.glBindTexture(normal.getTarget(), 0);
	}

	public static void main(String[] args) {
		ApplicationContext context = ApplicationContextProvider.getInstance()
				.getContext();
		context.run();
		RenderingContext mm = new Md5Test(context.getMainWindow());
		context.setCurrentRenderingContext(mm);

	}
}
